#!/bin/bash
POVER=2.4.1

# Set current directory to the one containing this script
cd "$(dirname "$0")"

# We assume we already executed make-distrib.bat on Windows
# and the PngOptimizer-$POVER directory already exists

cd PngOptimizer-$POVER || HandleFail
echo $PWD
mkdir -p linux/PngOptimizerCL

make -C ../../Projects/PngOptimizerCL/ CONFIG=release

function HandleFail
{
	tput setaf 1
	echo "="
	echo "=   FAILURE"
	echo "="
	tput sgr0
	exit 1
}

function HandleSuccess
{
	tput setaf 2
	echo "="
	echo "=   SUCCESS"
	echo "="
	tput sgr0
	exit 0	
}

###############################################################################
# PngOptimizerCL

cp -v "../../Projects/PngOptimizerCL/linux-release/PngOptimizerCL"  "linux/PngOptimizerCL/" || HandleFail
cp -v "../../Projects/PngOptimizerCL/Readme.txt"                    "linux/PngOptimizerCL/" || HandleFail
cp -v "../../Projects/PngOptimizerCL/License.txt"                   "linux/PngOptimizerCL/" || HandleFail
cp -v "../../Projects/PngOptimizerCL/Changelog.txt"                 "linux/PngOptimizerCL/" || HandleFail

###############################################################################
# Create PngOptimizerCL archive.
# We first need to fix file permissions that are not handled by
# the NTFS partition.
# To do that we copy the files to /tmp and change the permissions there
# before creating the tgz archive
TGZFILE="PngOptimizerCL-$POVER-linux-x64.tgz"

tput setaf 3
pushd .
rm -rf /tmp/pngotmp
mkdir /tmp/pngotmp || HandleFail
cp -avr linux/PngOptimizerCL /tmp/pngotmp || HandleFail
cd /tmp/pngotmp/
chmod 0755 PngOptimizerCL
chmod 0644 PngOptimizerCL/*
chmod 0755 PngOptimizerCL/PngOptimizerCL
echo "Creating $TGZFILE..."
tar -pczvf "$TGZFILE" "PngOptimizerCL" || HandleFail
popd
tput sgr0

cp -v "/tmp/pngotmp/$TGZFILE" "." || HandleFail

###############################################################################
# Sources
# That part should have been handled by the BAT version of the script on Windows,
# but in case we made some last minute changes and want to regenerate the source
# ZIP on Linux, just delete the source directory and the code below will do the job.

SRCDIR="PngOptimizer-sources"

if [ -d "$SRCDIR" ]; then
  echo "$SRCDIR directory already exists, exiting"
  HandleSuccess
  exit 0
fi

echo "$PWD/$SRCDIR does not exist, creating"
mkdir PngOptimizer-sources
hg archive -X ../../Projects/kbdfr_dk \
           -X ../../Projects/TestChu \
           -X ../../Projects/ViewPicNG \
           -X ../../Projects/PngOptimizerGtk \
           -X ../../SDK/chunet \
           -X ../../SDK/chugfx  PngOptimizer-sources
           
cp "../Readme-PngOptimizer-sources.txt" "PngOptimizer-sources/" || HandleFail

echo "Creating source file zip..."
# First we need to delete the archive if it exists because the zip tool
# add/merge/replace files by default which can lead to a content that
# does not reflect exactly the PngOptimizer-sources directory
rm -f "PngOptimizer-$POVER-sources.zip"
zip -r -q "PngOptimizer-$POVER-sources.zip" "PngOptimizer-sources" || HandleFail

HandleSuccess









