/////////////////////////////////////////////////////////////////////////////////////
// PngOptimizerDll - A DLL to get access to PngOptimizer optimization engine
// Copyright (C) 2012/2013 Hadrien Nilsson - psydk.org
// This DLL is distributed under the terms of the GNU LESSER GENERAL PUBLIC LICENSE
// See License.txt for the full license.
/////////////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#define PNGOPTIMIZERDLL_IMPL
#include "PngOptimizerDll.h"

////////////////////////////////////////////////////////////////////

::POEngine       g_engine;       // Optimizer engine as a global variable
CriticalSection  g_lock;         // Lock to protect global variables

////////////////////////////////////////////////////////////////////
// DLL <-> C# conversion routines

inline bool FromBool32(BOOL b32)
{
	return b32 != 0;
}

inline chustd::Color32 FromNetColor(DWORD dw)
{
	return chustd::Color32(dw);
}

inline DWORD ToNetColor(chustd::Color32 col)
{
	return col.value32;
}

PCWCH ToNetString(const chustd::String& str)
{
	if( str.IsEmpty() )
	{
		return NULL;
	}
	int count = (str.GetLength() + 1);
	WCHAR* p = (WCHAR*)::CoTaskMemAlloc(count * sizeof(WCHAR));
	wmemcpy(p, str.GetBuffer(), count);
	return p;
}

////////////////////////////////////////////////////////////////////

PNGOPTIMIZERDLL_API
BOOL PO_OptimizeFile(PCWCH filePath)
{
	BOOL ret;
	::POEngine::SingleOptiInfo soi;
	chustd::String filePathChu = filePath;

	g_lock.Enter();
	ret = g_engine.OptimizeSingleFile(filePathChu, "", soi);
	g_lock.Leave();
	return ret;
}

PNGOPTIMIZERDLL_API
BOOL PO_OptimizeFileMem(LPCVOID* image, INT imageSize,
                        LPVOID* result, INT resultCapacity, INT* pResultSize)
{
	BOOL ret;
	::POEngine::SingleOptiInfo soi;

	g_lock.Enter();
	ret = g_engine.OptimizeSingleFileMem((const uint8*)image, imageSize, (uint8*)result, resultCapacity, pResultSize);
	g_lock.Leave();
	return ret;
}

PNGOPTIMIZERDLL_API
PCWCH PO_GetLastErrorString()
{
	g_lock.Enter();
	String lastError = g_engine.GetLastErrorString();
	g_lock.Leave();
	return ToNetString(lastError);
}

PNGOPTIMIZERDLL_API
BOOL PO_SetSettings(const POSettings* s)
{
	// Convert to internal struct
	g_lock.Enter();
	g_engine.m_settings.backupOldPngFiles  = FromBool32(s->BackupOldPngFiles);
	g_engine.m_settings.keepInterlacing    = FromBool32(s->KeepInterlacing);
	g_engine.m_settings.avoidGreyWithSimpleTransparency = FromBool32(s->IE6Compatible);
	g_engine.m_settings.ignoreAnimatedGifs = FromBool32(s->SkipAnimatedGifs);
	g_engine.m_settings.keepFileDate       = FromBool32(s->KeepFileDate);

	g_engine.m_settings.bkgdOption = POChunkOption(s->BkgdOption);
	g_engine.m_settings.bkgdColor = FromNetColor(s->BkgdColor);

	g_engine.m_settings.textOption  = POChunkOption(s->TextOption);
	g_engine.m_settings.textKeyword = s->TextKeyword;
	g_engine.m_settings.textData = s->TextData;

	g_engine.m_settings.physOption = POChunkOption(s->PhysOption);
	g_engine.m_settings.physPpmX   = s->PhysPpmX;
	g_engine.m_settings.physPpmY   = s->PhysPpmY;
	g_lock.Leave();
	return TRUE;
}

PNGOPTIMIZERDLL_API BOOL PO_GetSettings(POSettings* s)
{
	// Convert to public struct
	g_lock.Enter();
	s->BackupOldPngFiles = g_engine.m_settings.backupOldPngFiles;
	s->KeepInterlacing   = g_engine.m_settings.keepInterlacing;
	s->IE6Compatible     = g_engine.m_settings.avoidGreyWithSimpleTransparency;
	s->SkipAnimatedGifs  = g_engine.m_settings.ignoreAnimatedGifs;
	s->KeepFileDate      = g_engine.m_settings.keepFileDate;

	s->BkgdOption  = g_engine.m_settings.bkgdOption;
	s->BkgdColor   = ToNetColor(g_engine.m_settings.bkgdColor);

	s->TextOption  = g_engine.m_settings.textOption;
	s->TextKeyword = ToNetString(g_engine.m_settings.textKeyword);
	s->TextData = ToNetString(g_engine.m_settings.textData);

	s->PhysOption = g_engine.m_settings.physOption;
	s->PhysPpmX   = g_engine.m_settings.physPpmX;
	s->PhysPpmY   = g_engine.m_settings.physPpmY;
	g_lock.Leave();
	return TRUE;
}

BOOL APIENTRY DllMain(HMODULE hModule, DWORD callReason, LPVOID)
{
	switch(callReason)
	{
	case DLL_PROCESS_ATTACH:
	case DLL_THREAD_ATTACH:
	case DLL_THREAD_DETACH:
	case DLL_PROCESS_DETACH:
		break;
	}
	return TRUE;
}

