///////////////////////////////////////////////////////////////////////////////
// This file is part of the PngOptimizerGtk application
// Copyright (C) 2002/2014 Hadrien Nilsson - psydk.org
//
// PngOptimizerGtk is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// Foobar is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Foobar; if not, write to the Free Software
// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
// 
///////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"

gboolean OnDragDataReceived(GtkWidget* widget, GdkDragContext* context, int x, int y,
                        GtkSelectionData* seldata, guint info, guint time,
                        gpointer userdata)
{
	Console::WriteLine("drag");
	return false;
}

gboolean OnDraw(GtkWidget* widget, cairo_t* cr, gpointer data)
{
	GdkRGBA color;

	guint width = gtk_widget_get_allocated_width(widget);
	guint height = gtk_widget_get_allocated_height(widget);
	cairo_arc(cr,
	         width / 2.0, height / 2.0,
	         MIN (width, height) / 2.0,
	         0, 2 * G_PI);

	gtk_style_context_get_color(gtk_widget_get_style_context(widget),
	                           GTK_STATE_FLAG_NORMAL,
	                           &color);
	gdk_cairo_set_source_rgba(cr, &color);

	cairo_fill(cr);
	return false;
}

int main(int argc, char** argv)
{
	gtk_init(&argc, &argv);

	GtkWidget* window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title(GTK_WINDOW(window), "PngOptimizer");
	gtk_window_set_default_size(GTK_WINDOW(window), 482, 300);
	gtk_window_set_position(GTK_WINDOW(window), GTK_WIN_POS_CENTER);

	GtkTargetEntry targets[] = {
		{ const_cast<char*>("text/uri-list"), GTK_TARGET_OTHER_APP, 0xb00b00 }
	};

	gtk_drag_dest_set(window, GTK_DEST_DEFAULT_ALL,
		targets, ARRAY_SIZE(targets), GDK_ACTION_COPY);

    g_signal_connect(window, "drag_data_received",
                     G_CALLBACK(OnDragDataReceived), NULL);


    GtkWidget* drawingArea = gtk_drawing_area_new();
    gtk_widget_set_size_request(drawingArea, 800, 600);
    gtk_container_add(GTK_CONTAINER(window), drawingArea);
    g_signal_connect(drawingArea, "draw", G_CALLBACK(OnDraw), NULL);

	g_signal_connect(window, "destroy", G_CALLBACK(gtk_main_quit), NULL);

	gtk_widget_show(window);

	gtk_main();
	
	return 0;
}
